@@ -198,11 +198,6 @@ vcov.emmGrid = function(object, ...) {
 #' \item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
 #' argument in \code{\link{summary.emmGrid}}.}
 #' 
-#' \item{\code{estType}}{(\code{character}) is the type of the estimate. It
-#' should match one of \samp{c("prediction", "contrast", "pairs")}. This is used
-#' along with \code{"adjust"} to determine appropriate adjustments to P values
-#' and confidence intervals.}
-#' 
 #' \item{\code{famSize}}{(integer) is the number of means involved in a family of
 #' inferences; used in Tukey adjustment}
 #' 
@@ -234,6 +229,16 @@ vcov.emmGrid = function(object, ...) {
 #' \code{"link"} (with synonyms \code{"lp"} and \code{"linear"}), or
 #' \code{"response"}.}
 #' 
+#' \item{\code{bias.adjust}, \code{frequentist}}{(character) These
+#' are used by \code{summary} if the value of these arguments are not specified.}
+#' 
+#' \item{\code{estType}}{(\code{character}) is used internally to determine 
+#' what \code{adjust} methods are appropriate. It should match one of 
+#' \samp{c("prediction", "contrast", "pairs")}. As an example of why this is needed,
+#' the Tukey adjustment should only be used for pairwise comparisons 
+#' (\code{estType = "pairs"}); if \code{estType} is some other string, Tukey
+#' adjustments are not allowed.}
+#' 
 #' \item{\code{avgd.over}}{(\code{character)} vector) are the names of the 
 #' variables whose levels are averaged over in obtaining marginal averages of 
 #' predictions, i.e., estimated marginal means. Changing this might produce a 
@@ -284,12 +289,9 @@ vcov.emmGrid = function(object, ...) {
 #' emmeans(mypigs.rg, "source")
 update.emmGrid = function(object, ..., silent = FALSE) {
     args = list(...)
-    valid.misc = c("adjust","alpha","avgd.over","by.vars","delta","df",
-                   "initMesg","estName","estType","famSize","infer","inv.lbl",
-                   "level","methDesc","nesting","null","predict.type","pri.vars"
-                   ,"side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg")
+    # see .valid.misc below this function for list of legal options
     valid.slots = slotNames(object)
-    valid.choices = union(valid.misc, valid.slots)
+    valid.choices = union(.valid.misc, valid.slots)
     misc = object@misc
     for (nm in names(args)) {
         fullname = try(match.arg(nm, valid.choices), silent=TRUE)
@@ -349,6 +351,13 @@ update.emmGrid = function(object, ..., silent = FALSE) {
     object
 }
 
+### List of valid strings to match in update() ###
+.valid.misc = c("adjust","alpha","avgd.over","bias.adjust","by.vars","delta","df",
+               "initMesg","estName","estType","famSize","frequentist","infer","inv.lbl",
+               "level","methDesc","nesting","null","predict.type","pri.vars"
+               ,"side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg")
+
+
 #' Set or change emmeans options
 #'
 #' Use \code{emm_options} to set or change various options that are used in
@@ -714,7 +723,9 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
                else                     make.link(transform)
         bounds = range(link$linkinv(c(-1e6, -100, -1, 0, 1, 100, 1e6)))
         nas = which(is.na(object@bhat)) # already NA
-        incl = which((object@bhat > bounds[1]) & (object@bhat) < bounds[2])
+        incl = vincl = which((object@bhat > bounds[1]) & (object@bhat < bounds[2]))
+        if (length(nas) > 0) 
+            vincl = which((object@bhat[-nas] > bounds[1]) & (object@bhat[-nas] < bounds[2]))
         negs = setdiff(seq_along(object@bhat), incl)
         if (length(negs) > 0) {
             message("Invalid response predictions are flagged as non-estimable")
@@ -726,7 +737,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
         Vee = object@V
         if(length(incl) > 0) {
             D = .diag(1/link$mu.eta(object@bhat[incl]))
-            object@V = D %*% tcrossprod(Vee[incl, incl, drop = FALSE], D)
+            object@V = D %*% tcrossprod(Vee[vincl, vincl, drop = FALSE], D)
         }
         if (!is.na(PB[1])) {
             PB[PB <= 0] = NA
