@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2019 Russell V. Lenth                                #
+#    Copyright (c) 2012-2020 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -85,7 +85,7 @@
 #' @param type Character value. If provided, this is saved as the
 #'   \code{"predict.type"} setting. See \code{\link{update.emmGrid}} and the
 #'   section below on prediction types and transformations.
-#' @param transform Character value. If other than \code{"none"}, the reference
+#' @param transform Character, logical, or list. If non-missing, the reference
 #'   grid is reconstructed via \code{\link{regrid}} with the given
 #'   \code{transform} argument. See the section below on prediction types and
 #'   transformations.
@@ -331,13 +331,19 @@
 #' # Silly illustration of how to use 'mult.levs' to make comb's of two factors
 #' ref_grid(MOats.lm, mult.levs = list(T=LETTERS[1:2], U=letters[1:2]))
 #' 
+#' # Using 'params'
+#' require("splines")
+#' my.knots = c(2.5, 3, 3.5)
+#' mod = lm(Sepal.Length ~ Species * ns(Sepal.Width, knots = my.knots), data = iris)
+#' ## my.knots is not a predictor, so need to name it in 'params'
+#' ref_grid(mod, params = "my.knots") 
+#' 
 ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("cov.keep"),
                      mult.names, mult.levs, 
                      options = get_emm_option("ref_grid"), data, df, type, 
-                     transform = c("none", "response", "mu", "unlink", "log"), 
-                     nesting, offset, sigma, ...) 
+                     transform, nesting, offset, sigma, ...) 
 {
-    transform = match.arg(transform)
+    ### transform = match.arg(transform)
     if (!missing(df)) {
         if(is.null(options)) options = list()
         options$df = df
@@ -745,7 +751,7 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         result@misc$postGridHook = NULL
         result = hook(result, ...)
     }
-    if(transform != "none")
+    if(!missing(transform))
         result = regrid(result, transform = transform, sigma = sigma, ...)
     
     .save.ref_grid(result)
